__title__ = "videofingerprint"
__description__ = (
    "Relative Luminance gradient and Dominant Color of Frames (RLGDCF) "
    + "based Digital Video Fingerprinting. "
)

__url__ = "https://akamhy.github.io/videofingerprint/"
__version__ = "0.0.1"
__status__ = "production"
__author__ = "Akash Mahanty"
__author_email__ = "akamhy@yahoo.com"
__license__ = "MIT"
__copyright__ = "Copyright 2022 Akash Mahanty"
